@@ -24,6 +24,12 @@ Created on May 22, 2012
 '''
 history = \
 """
+0.4.1 - 30/05/2013
+        Re-organised code so that external modules have intrinsic functions which
+        are USED by PyGS but not automatically classes - makes it easier to trace
+        bugs.
+        Re-wrote lumfunc code.
+        
 0.4.0 - 15/05/13 (final)
         Made DFT multi-threaded for performance
         Modified Power Spectra functions to be easier to call and takes weights        
@@ -117,18 +123,18 @@ except:
 
 
 #Modules in PyGS
-from LuminosityFunction import LuminosityFunction as lf
-from SimplePlots import SimplePlots as sp
-from PowerSpectra import PowerSpectra as ps
-from Correlations import Correlations as corr
+import LuminosityFunction as lf
+#from SimplePlots import SimplePlots as sp
+import PowerSpectra as ps
+#from Correlations import Correlations as corr
 import utils
-from SelectionFunction import SelectionFunction as sf
-from Cosmology import Cosmology as csm
+import SelectionFunction as sf
+#from Cosmology import Cosmology as csm
 
 #============================================
 # THE PYGS CLASS
 #============================================
-class Import(lf, sp, ps, corr, sf, csm):
+class Import(object):
 
     def __init__(self,
                  filename,  #Filename of survey to be read in (absolute path)
@@ -137,7 +143,7 @@ class Import(lf, sp, ps, corr, sf, csm):
                  sample_name=None,  #Which sample to use (can always be 'all', otherwise will make one)
                  sample_cuts=None,  #Cuts to apply to the sample
                  clobber=None,  #Whether to automatically overwrite previous project folders
-                 ):
+                 verbose = True):
         '''
         Imports the survey and creates a project directory and internal structure. 
         
@@ -202,7 +208,7 @@ class Import(lf, sp, ps, corr, sf, csm):
         self.sample_name = sample_name  #Which sample to use (can always be 'all', otherwise will make one)
         self.sample_cuts = sample_cuts  #Cuts to apply to the sample
         self._clobber = clobber  #Whether to automatically overwrite previous project folders
-
+        self.verbose = verbose
         self.plottype = 'png'
 
         #The columns wanted from the original file
@@ -243,11 +249,11 @@ class Import(lf, sp, ps, corr, sf, csm):
         if 'mag_r' in self.survey_data.columns and 'mag_g' in self.survey_data.columns:
             #Find the g-r colour and K-Correction to find Abs_Mag and luminosity.
             gr = self.survey_data["mag_g"] - self.survey_data["mag_r"]
-            kcor = self._KCorrect(gr)
+            self.survey_data['kcorr'] = self._KCorrect(gr)
 
-            self.survey_data["absmag"] = self._AbsoluteMag(lum_dist, kcor)
+            self.survey_data["absmag"] = self._AbsoluteMag(lum_dist, self.survey_data['kcorr'])
             self.survey_data['lum'] = self._LuminosityFromAbsMag(self.survey_data['absmag'])
-
+            
         elif 'mag_g' in self.survey_data.dtype.names:
             #calculate absmag and luminosities without K-Correction
             self.survey_data["absmag"] = self._AbsoluteMag(lum_dist)
@@ -468,7 +474,7 @@ class Import(lf, sp, ps, corr, sf, csm):
         """
         store = pandas.HDFStore(filename)
         if query:
-            self.survey_data = store.select['survey', query]
+            self.survey_data = store.select('survey', query)
         else:
             self.survey_data = store['survey']
 
@@ -569,23 +575,24 @@ class Import(lf, sp, ps, corr, sf, csm):
         sample_cuts = pandas.DataFrame(index=["min", "max"])
 
         for col in self._raw_columns:
-
-            try:
-                sample_cuts[col][0] = input("What is the minimum " + col + " preferred? ["
-                                                  + str(min(self.survey_data[col])) + ','
-                                                  + str(max(self.survey_data[col])) + "]: ")
-            except SyntaxError:
-                sample_cuts[col][0] = None
-                print ""
-
-            try:
-                self.sample_cuts[col][1] = input("What is the maximum " + col + " preferred? ["
-                                                  + str(min(self.survey_data[col])) + ','
-                                                  + str(max(self.survey_data[col])) + "]: ")
-            except SyntaxError:
-                sample_cuts[col][1] = None
-                print ""
-
+            if col != 'id':
+                try:
+                    min_input = input("What is the minimum " + col + " preferred? ["
+                                                      + str(min(self.survey_data[col])) + ','
+                                                      + str(max(self.survey_data[col])) + "]: ")
+                except SyntaxError:
+                    min_input = None
+                    print ""
+    
+                try:
+                    max_input = input("What is the maximum " + col + " preferred? ["
+                                                      + str(min(self.survey_data[col])) + ','
+                                                      + str(max(self.survey_data[col])) + "]: ")
+                except SyntaxError:
+                    max_input = None
+                    print ""
+    
+                sample_cuts[col] = [min_input,max_input]
         return sample_cuts
 
 
@@ -613,5 +620,259 @@ class Import(lf, sp, ps, corr, sf, csm):
 
 
 
+#==============================================================================
+#   Now we start individual "modules" within the class
+#==============================================================================
+    
+# --------------- Luminosity Funtion methods ----------------------------------
+    def lum_func(Np,min_acceptable=0,eps=0.01):
+        """
+        Calculates the luminosity function of galaxies in Np bins.
+        Uses EEP method only currently (that may be expanded)
+        """
+        
+        phi, iteration = lf.lum_func(absmag = self.survey_data['absmag'],
+                                     mag_r = self.survey_data['mag_r'],
+                                     c_dist = self.survey_data['c_dist'],
+                                     k_corr = self.survey_data['k_corr'],
+                                     Np = Np,
+                                     min_acceptable=minacceptable,
+                                     eps=eps,
+                                     verbose=self.verbose
+                                     )
+        self.LF = phi[iteration]
+        self.LF_iterations = phi
+        
+        
+# -------------- PowerSpectra methods -----------------------------------------
+        
+    def power_spec(quantity="redshift", bins=None, min_wave=0.0001, 
+                      max_wave=0.02, n_waves = 1000,pad=1, in_wavelength=True, 
+                      weight=True,slow=True):
+        """
+        Computes the 1D power spectrum and plots it.
+    
+        This method uses the numpy fft algorithm to calculate the real to real 
+        DFT of any quantity in the GS data, then converts this to a power
+        spectrum (merely squares it). As such, it needs what I call 'density 
+        values' as its input, which here are merely histogram values. It will 
+        only calculate as many DFT points as there are bins in the histogram, 
+        which is fine for frequency-based plots, but makes wavelength (inverse 
+        frequency) plots very angular. To increase the number of points you can
+        pad the original data with zeroes on either end, but even this does not
+        ensure a smooth wavelength plot. To get a very nice looking wavelength 
+        plot, use PowerSpec_1d_slow().
+        
+        
+        INPUT PARAMETERS:
+        quantity    [redshift]    : the quantity type used for the dft. Generally 'redshift', 'c_dist', 'ra' or 'dec'
+        bins        [None]        : the number of bins to use in the histogram to get 'densities'. 
+                                    If none, an 'optimal' bin number is used.
+        min_wave    [0.0001]      : The minimum wavelength to plot.
+        max_wave    [0.02]        : The maximum wavelength to plot
+        pad         [1]           : How much the original data length is multiplied by (zeroes padded in the extra space).
+                                    The higher the padding, the longer the calculation, but the better the resolution.
+        wavelength  [True]        : Whether to plot the power against wavelength. If False, plots against frequency.
+        weight      [None]        :  An optional setting allowing a further weight to be applied to the bin densities before calculation.
+                                    0 - No extra weighting (each bin will be its number)
+                                    'weights_1' - Inverse of selection function (simple)
+                                    'weights_2' - Inverse of selection function +1 (simple)
+        
+        OUTPUT PARAMETERS:
+        ps                        : array of power spectrum values
+        freq                      : array of frequencies
+        wvl                       : array of wavelengths
+        """
+        
+        #Set some default options
+        if quantity is 'c_dist' and max_wave < 2:
+            min_wave = 10
+            max_wave = 180
+        elif quantity is 'ra' and min_wave == 0.0001 and max_wave == 0.02:
+            min_wave = 0.01
+            max_wave = 2 * np.pi
+        elif quantity is 'dec' and min_wave == 0.0001 and max_wave == 0.02:
+            min_wave = 0.01
+            max_wave = np.pi
+            
+        if quantity != 'redshift' and quantity != 'c_dist':
+            weight = False
+            
+        #Get weights if not already in the data
+        if weight:
+            if 'weight' not in self.survey_data.columns:
+                self.do_selection_and_weighting()
+            weight = self.survey_data['weight']
+            
+        #Do calculations
+        if not slow:
+            power, freq, wavelength = ps.power_spec_1d_fast(quantity = self.survey_data['quantity'],
+                                                  bins=bins, 
+                                                  min_wave=min_wave,
+                                                  max_wave=max_wave, 
+                                                  pad=pad,  
+                                                  weight=weight)
+        else:
+            power, freq, wavelength = ps.power_spec_1d_slow(quantity = self.survey_data['quantity'],
+                                                  bins=bins, 
+                                                  min_wave=min_wave,
+                                                  max_wave=max_wave, 
+                                                  n_waves = n_waves, 
+                                                  weight=weight)
+        #Plot the function                                                  
+        plt.clf()
+        plt.title("1D P.S. for " + quantity + " of sample " + self.sample_name.partition('.')[0] + \
+                  " from " + self.survey_name)
+        if in_wavelength:
+            plt.plot(wavelength[(wavelength < max_wave) & (wavelength > min_wave)], 
+                                power[(wavelength < max_wave) & (wavelength > min_wave)], 'b-')
+            if quantity == 'redshift':
+                plt.xlabel("Wavelength, (z)")
+            elif quantity == 'c_dist':
+                plt.xlabel(r'Wavelength, Mpc $h^{-1}$s')
+        else:
+            plt.plot(freq[(wavelength < max_wave) & (wavelength > min_wave)] * 2 * np.pi,
+                          power[(wavelength < max_wave) & (wavelength > min_wave)], 'b-')
+            plt.xlabel("Frequency (radians)")
+        plt.ylabel("Power")
+        if in_wavelength:
+            plt.savefig(self._dirs['PowerSpecs_1D'] + quantity + '_bins' + str(bins) + '_wavelength.eps')
+        else:
+            plt.savefig(self._dirs['PowerSpecs_1D'] + quantity + '_bins' + str(bins) + '_frequency.eps')
+        
+        plt.show()
+        
+#==============================================================================
+#  Selection Function Routines 
+#==============================================================================
+        
+    def do_selection_and_weighting(self,bins=None):
+        """
+        Performs selection function fitting based on simple selection form only
+        """
+        
+        A,g,t = sf.fit_selection_simple(self.survey_data['redshift'],
+                                     bins=bins,verbose=self.verbose)
+        
+        self.survey_data['weight'] = 1 / sf.selection_simple(self.survey_data['redshift'], 
+                                                             A=A,g=g,t=t)
+                                                            
+        self.selection_params = {'A':A,'g':g,'t':t}
+
+    def create_radial_selection(self,N=None):
+        """
+        Creates a random catalogue of particles based on a defined selection function. Only radial co-ords.
+        
+        Ouput: radii :: length N vector (default length of survey) of radii        
+        """
+
+        z_min = self.survey_data['redshift'].min
+        z_max = self.survey_data['redshift'].max           
+
+        #Generate some random numbers
+        if N is None:
+            N = self.N                                         
+            
+        radii = sf.create_radial_selection(z_min,z_max, N)
+        
+        return radii
+        
+#==============================================================================
+# Simple Plots 
+#==============================================================================
+
+    def PlotHist(self, quantity="redshift", bins=None, weight=None):
+        """
+        Plots a Histogram of the data for any quantity
+        """
+
+        if not bins:
+            bins = utils.FD_bins(self.survey_data[quantity])
 
 
+        plt.clf()
+        plt.title("Histogram of " + quantity + " values for Sample " + self.sample_name + " from " + self.survey_name)
+        if not weight:
+            plt.hist(self.survey_data[quantity], bins)
+        else:
+            plt.hist(self.survey_data[quantity], bins, weights=self.survey_data[weight])
+
+        plt.xlabel(quantity)
+        plt.ylabel("Number")
+        plt.savefig(self._dirs['SurveyPlots'] + 'Histogram_' + quantity + '_' + str(bins) + "bins.eps")
+        plt.show()
+
+    def PlotPolarDot(self, distance=False):
+        """
+        Makes a polar plot of the data, TODO: with an optional filter condition
+            
+        If distance is true, plots against comoving distance rather than redshift.
+        """
+
+        plt.clf()
+        plt.title("Polar Plot of sample " + self.sample_name + " from " + self.survey_name)
+
+        if not distance:
+            plt.polar(self.survey_data['ra'], self.survey_data['redshift'], '.', markersize=1)
+            plt.savefig(self._dirs['SurveyPlots'] + 'PolarPlot_Redshift.eps')
+        else:
+            plt.polar(self.survey_data['ra'], self.survey_data['c_dist'], '.', markersize=1)
+            plt.savefig(self._dirs['SurveyPlots'] + 'PolarPlot_Distance.eps')
+        plt.show()
+
+    def PlotPolarSmooth(self, bins=None):
+        """
+        Makes a polar plot of the data smoothed by a filter., TODO: with an optional filter condition
+            
+        If distance is true, plots against comoving distance rather than redshift.
+        """
+        x_p = self.survey_data['c_dist'] * np.cos(self.survey_data['ra'])
+        y_p = self.survey_data['c_dist'] * np.sin(self.survey_data['ra'])
+
+        if not bins:
+            bins = np.ceil(np.sqrt(self.N / 10000))
+            if bins < 30:
+                bins = 30
+
+        smooth = np.histogram2d(x_p, y_p, ((bins, bins)))[0]
+
+        plt.clf()
+        plt.title("Smoothed Polar Plot of sample " + self.sample_name + " from " + self.survey_name)
+        plt.imshow(smooth, interpolation="Gaussian", aspect='equal')
+        plt.tick_params(axis='both', labelbottom=False, labelleft=False)
+        plt.savefig(self._dirs['SurveyPlots'] + 'PolarPlot_Smooth.eps')
+        plt.show()
+
+    def PlotSkyDot(self):
+        """
+        Makes a 2-D flat sky-plot of the data points.
+        """
+
+        plt.clf()
+        plt.title("Sky Plot of sample " + self.sample_name + " from " + self.survey_name)
+        plt.plot(self.survey_data['ra'], self.survey_data['dec'], '.', markersize=1)
+        plt.xlabel("Right Ascension")
+        plt.ylabel("Declination")
+        plt.savefig(self._dirs['SurveyPlots'] + 'SkyPlot.eps')
+        plt.show()
+
+    def PlotSkySmooth(self, bins=None):
+        """
+        Makes a 2-D flat sky-plot of the data points, smoothed by Gaussian filter.
+        """
+        plt.clf()
+        plt.title("Smoothed Sky Plot of sample " + self.sample_name + " from " + self.survey_name)
+
+        if not bins:
+            bins = np.ceil(np.sqrt(self.N / 10000))
+            if bins < 30:
+                bins = 30
+
+        smooth = np.histogram2d(self.survey_data['ra'], self.survey_data['dec'], ((bins, bins)))[0]
+
+        plt.clf()
+        plt.imshow(smooth, interpolation=None, aspect='equal', extent=(np.min(self.survey_data['ra']),
+                    np.max(self.survey_data['ra']), np.min(self.survey_data['dec']), np.max(self.survey_data['dec'])))
+        plt.tick_params(axis='both')
+        plt.savefig(self._dirs['SurveyPlots'] + 'SkyPlot_Smooth.eps')
+        plt.show()
